/*
 * Telephone.java
 * Purpose:  Provides Person storage and validation for Assignment 2
 *
 * By:  Ron Ritchey
 * Oct 7, 2007
 *
 *
 */

import java.util.logging.*;
import java.util.regex.*;
import java.io.*;

public class Telephone implements Comparable, Serializable, ObjectInputValidation {
	String CountryCode=new String("");
	String AreaCode=new String("");
	String LocalNumber=new String("");
	String s=new String("");

	// This class holds Telephone numbers.
	// It will perform regex validation of phone numbers before accepting them.
	// It should accept all US and international phone numbers

	public final static String telLocal = "[1-9][0-9 ,.\\-/]{4,8}$";
	public final static String telAccess = "((011[ \\-,./])|(00[ \\-,./]?)|(\\+))";
	public final static String telTrunk = "([01][ \\-,./]?)?";
	public final static String telCountry = "(([1-9][0-9]{0,2})|(\\([1-9][0-9]{0,2}\\)))[ ,.\\-/]?";
	public final static String telAreaCode = "\\([1-9][0-9]{0,5}\\)[ ,.\\-/]?";
	public final static String FullTelPattern = "(("+telAccess+telCountry+telAreaCode+")|("+telTrunk+telAreaCode+"))?"+telLocal;

	private static String validate(String input, String pattern) throws InvalidInputException {
		Pattern p = Pattern.compile(pattern);
		Matcher m = p.matcher(input);

		if (!m.find()) throw new InvalidInputException();

		return m.group();
	}

	private static Logger logger =
	  Logger.getLogger(Person.class.getName());

	public Telephone(Telephone t) throws InvalidTelephoneException {
		try { validate(t.getNumber(), FullTelPattern); }
		catch(InvalidInputException e) { logger.fine("Received invalid telephone number"); throw new InvalidTelephoneException(); }

		CountryCode = new String(t.CountryCode);
		AreaCode = new String(t.AreaCode);
		LocalNumber = new String(t.LocalNumber);
	}

	public Telephone(String tel) throws InvalidTelephoneException {

		// Make sure we got a good telephone #
		try { validate(tel, FullTelPattern); }
		catch(InvalidInputException e) { logger.fine("Received invalid telephone number"); throw new InvalidTelephoneException(); }


		// Creating string that we will consume as we parse it
		s = tel;

		// Check for country code
		try {
			CountryCode = validate(s, "^"+telAccess+telCountry);

			// No exception so this must have a country code
			s = s.substring(CountryCode.length());
			CountryCode = CountryCode.replaceAll("^00|^011|[\\s,.+\\-/\\(\\)]", ""); // Remove all formatting characters
		} catch (InvalidInputException e) {
			//
			CountryCode = ""; // No Country Code detected
		}

		// Check for area code
		try {
			AreaCode = validate(s, telTrunk+telAreaCode);

			// No exception so this must have an area code
			s = s.substring(AreaCode.length());

			// Remove any trunk characters
			AreaCode = validate(AreaCode, telAreaCode+"$");
			AreaCode = AreaCode.replaceAll("[ ,.\\-/\\(\\)]","");
		} catch (InvalidInputException e) {

			// If areacode has a value but an InvalidInputException was thrown, there was a problem removing the trunk
			if (AreaCode.length() > 0) {
				logger.fine("Problem removing trunk from areacode");
				throw new InvalidTelephoneException();
			} else AreaCode = ""; // No area code detected

		}

		//Check for local
		try {

			s = s.trim();

			String sTmp = new String(validate(s, telLocal));

			// No exception so a local value has been found

			// if lengths don't match an area code may be embedded
			if (sTmp.length() != s.length()) {

				// An area code already exists so this local may be bad
				if (!AreaCode.equals("")) {
					logger.fine("Number appears to have an invalid local value");
					throw new InvalidTelephoneException();
				}

				// Don't have area code yet so lets look for one

				// Eat any initial separator
				sTmp = s;
				if (sTmp.matches("^[ ,.\\-/\\(\\)]")) {
					sTmp = sTmp.substring(1);
					s = s.substring(1);
				}

				//Collect values up to first separator.

				//Note: if the next validate throws exception then LocalNumber will not have been set
				//This will result in an InvalidTelephoneException being thrown to the calling method

				AreaCode = validate(sTmp, "[0-9]{1,5}[ ,.\\-/\\(\\)]"); // Got AreaCode (maybe)
				s = s.substring(AreaCode.length());
				AreaCode = AreaCode.replaceAll("[ ,.\\-/\\(\\)]", "");

				LocalNumber = validate(s, "^"+telLocal); // Verify the remainder is still a good local number

			} else LocalNumber = s;


		} catch(InvalidInputException e) {
			// If no local number, this number is invalid
			logger.fine("Number was invalid even though it passed initial pattern check");
			throw new InvalidTelephoneException();
		}

		logger.finer("Successful initialization of telephone number "+CountryCode+" " + AreaCode + " " + LocalNumber);


	} // End of Telephone(string tel)

	public String getCountryCode() { return CountryCode; }
	public String getAreaCode() { return AreaCode; }
	public String getLocalNumber() { return LocalNumber; }

	public String getTelNr() { return "+" + CountryCode + " (" + AreaCode + ") " + LocalNumber; }
	public String getNumber() {
		String build = new String("");

		if (!CountryCode.equals("")) build = build + "+" + CountryCode + " ";
		if (!AreaCode.equals("")) build = build + "(" + AreaCode + ") ";
		if (!LocalNumber.equals("")) build = build + LocalNumber;

		return build;
	}

	public int compareTo(Object anotherTel) throws ClassCastException {
		if (!(anotherTel instanceof Telephone))
			throw new ClassCastException("Telephone object expected");

		if(CountryCode.compareTo( ((Telephone) anotherTel).getCountryCode() ) == 0) {
			if (AreaCode.compareTo( ((Telephone) anotherTel).getAreaCode() ) == 0) {
				return (LocalNumber.compareTo( ((Telephone) anotherTel).getLocalNumber()) );
			}
			else return AreaCode.compareTo( ((Telephone) anotherTel).getAreaCode() );
		}
		else return CountryCode.compareTo( ((Telephone) anotherTel).getCountryCode() );
	}

	private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
		ois.registerValidation(this, 0);
		ois.defaultReadObject();

	}

	public void validateObject() throws InvalidObjectException {
		try {
			Telephone t = new Telephone(getNumber());
		} catch (InvalidTelephoneException e) {
			throw new InvalidObjectException("Invalid telephone value in stream");
		}
	}
}
